/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.util.Set;
import java.util.TreeSet;

/**
 * Utility class used to help build terms.
 * 
 * @author Jacek Koleyski
 */
public class Utils {
	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Creates a term from a line.
	 * 
	 * @param prefix
	 *            prefix to be added to the term
	 * @param line
	 *            the line of text
	 * @param nr
	 *            line number
	 * 
	 * @return the created ITerm
	 * 
	 * @throws DictionaryCorruptedException
	 *             if the dictionary file is corrupted
	 */
	public static ITerm createTerm(String prefix, String line, int nr)
			throws DictionaryCorruptedException {
		ITerm term = null;

		try {
			String[] result = line.split(Reminder.termSeparator);

			if (result.length < 2) {
				String message;

				if (nr == -1) {
					message = line;
				} else {
					message = Reminder.lineNrString + nr + " : " + line;
				}

				throw new DictionaryCorruptedException(message);
			}

			String word = result[0].trim();
			String rest = result[1].trim();

			if ((word == null) || word.equals("") || (rest == null)
					|| rest.equals("")) {
				String message;

				if (nr == -1) {
					message = line;
				} else {
					message = Reminder.lineNrString + nr + " : " + line;
				}

				throw new DictionaryCorruptedException(message);
			}

			String[] translations = rest.split(Reminder.translationSeparator);

			Set<String> trimmedSet = new TreeSet<String>();

			if (translations.length > 1) {
				// add all translations to a set in order to sort them and
				// remove possible duplicates
				for (int i = 0; i < translations.length; i++) {
					trimmedSet.add(translations[i].trim());
				}

				translations = (trimmedSet
						.toArray(new String[trimmedSet.size()]));
			}

			if (prefix == null)
				term = new CompositeTerm(word, translations);
			else
				term = new PrefixTerm(word, translations, prefix);
		} catch (RuntimeException e) {
			String message;

			if (nr == -1) {
				message = line;
			} else {
				message = Reminder.lineNrString + nr + " : " + line;
			}

			throw new DictionaryCorruptedException(message);
		}

		return term;
	}

	/**
	 * Creates a term from a line, without prefix. Delegates work to the method
	 * taking prefix as an argument.
	 * 
	 * @param line
	 *            the line of text
	 * @param nr
	 *            line number
	 * 
	 * @return the created ITerm
	 * 
	 * @throws DictionaryCorruptedException
	 *             if the dictionary file is corrupted
	 */
	public static ITerm createTerm(String line, int nr)
			throws DictionaryCorruptedException {
		return createTerm(null, line, nr);
	}
}
